"""
sets up configuration for Sphinx docs
"""
from pathlib import Path
import shutil


def make_dir_if_not_exists(path: Path):
    if not path.exists():
        Path.mkdir(path)


def make_script_rst_(script, source_dir):
    n_chars = len("Module: mod:") + len(script) + 2
    chars_line = '='*n_chars
    rst_contents = f"""Module :mod:`{script}`
{chars_line}

.. automodule:: {script}
   :members:
   :undoc-members:
   :show-inheritance:
    """
    with open(f"{source_dir}/{script}.rst", "w") as f:
        f.write(rst_contents)


def main():
    package_name = Path.cwd().stem

    with open("modules_todoc.txt", "r") as f:
        scripts = [l.strip('\n') for l in f.readlines()]

    CURDIR = Path.cwd()

    CONF_SPHINX = CURDIR / "sphinx_conf_bs.py"
    MAKE_SPHINX = CURDIR / "Sphinx_Makefile"

    docs_dir = CURDIR / "docs"
    make_dir_if_not_exists(docs_dir)
    source_dir = docs_dir / "source"
    make_dir_if_not_exists(source_dir)
    build_dir = docs_dir / "build"
    make_dir_if_not_exists(build_dir)

    shutil.copy(CONF_SPHINX, source_dir / "conf.py")
    conf_filename = str(source_dir / "conf.py")
    with open(conf_filename, "r") as f:
        contents_conf = f.read()
    contents_updated = contents_conf.replace(
        "name_of_directory_of_package", package_name)
    with open(conf_filename, "w") as f:
        f.write(contents_updated)

    shutil.copy(MAKE_SPHINX, docs_dir / "Makefile")

    shutil.copy("README.rst", source_dir)

    index_rst_file = source_dir / "index.rst"

    index_file_contents = f"Documentation for package {package_name}\n"
    n_chars = len(index_file_contents)
    chars_line = '='*n_chars
    index_file_contents += chars_line + '\n'

    with open("title.txt", "r") as f:
        title = f.read()
    index_file_contents += ("\n" + title + "\n")

    index_file_contents += """
    

.. toctree::
   :maxdepth: 2

"""

    index_file_contents += '\n   README'

    for script in scripts:
        index_file_contents += ('\n   ' + script)
        make_script_rst_(script, source_dir)

    index_file_contents += '\n'

    with open(f"{index_rst_file}", "w") as f:
        f.write(index_file_contents)

    # create script to build docs
    build_docs_contents = "import os\n\n" + "os.system('make clean')\n" \
        + "os.system('make html')\n"
    with open(docs_dir / "build_docs.py", "w") as f:
        f.write(build_docs_contents)


if __name__ == "__main__":
    main()
